import java.util.ArrayList;

/**
 * SnowflakeList.
 * 
 * Verwaltet eine Menge an Schneeflocken. Die Hauptmethode fall lässt sie
 * schrittweise von einem virtuellen Himmel fallen.
 * 
 * @author Stephan Bauer
 * @author Thomas Gradisnik
 * @version 1.0
 * 
 */

public class SnowflakeList {

    /**  Liste aller Schneeflocken */
    ArrayList<Snowflake> flakes;

    private int width;
    private int height;
    private int lifetime;

    /**
     * Konstruktor.
     * 
     * Initialisiert eine neue Liste mit einer Initialanzahl von Schneeflocken.
     * 
     * @param amount
     *            Initialanzahl von Schneeflocken
     * @param width
     *            Breite des Konsolenfensters
     * @param height
     *            Höhe des Konsolenfensters
     * @param lifetime
     *            Lebensdauer einer Schneeflocke
     */
    public SnowflakeList(int amount, int width, int height, int lifetime) {

	this.flakes = new ArrayList<Snowflake>();

	this.width = width;
	this.height = height;
	this.lifetime = lifetime;

	for (int i = 0; i < amount; i++) {
	    addFlake(true);
	}

    }

    /**
     * Fügt der Liste eine neue Schneeflocke hinzu.
     * 
     * Die Anfangsposition in x-Richtung und die Lebenszeit einer neuen
     * Schneeflocke wird zufällig gewählt, in y-Richtung entweder mit fester
     * Startposition an der Oberkante des Fensters oder zufällig.
     * 
     * @param randomYEnabled
     *            Zufällige Startposition in y-Richtung.
     */
    private void addFlake(boolean randomYEnabled) {

	int randomX, randomY, randomT;

	randomT = (int) ((0.5 + Math.random()) * lifetime);
	randomX = (int) (Math.random() * width);
	randomY = (randomYEnabled) ? ((int) (Math.random() * (-1) * height)) : (-1);

	char symbol;
	if (Math.random() > 0.3) {
	    symbol = '*';
	} else if (Math.random() > 0.6) {
	    symbol = '+';
	} else {
	    symbol = '.';
	}
	flakes.add(new Snowflake(randomX, randomY, randomT, symbol));

    }

    /**
     * Lässt Schneeflocken vom virtuellen Himmel fallen.
     * 
     * Alle Schneeflocken werden um eine Zeile nach unten und dabei zufällig
     * auch nach links oder rechts verschoben. Geschmolzene Schneeflocken werden
     * aus der Liste entfernt und durch neue ersetzt.
     */
    public void fall() {

	for (int i = 0; i < flakes.size(); i++) {

	    // Nach unten und zufällig nach links oder rechts verschieben
	    flakes.get(i).move((int) Math.round(Math.random() * 2) - 1, 1);

	    // Gewichtung: Schneeflocken am Rand schmelzen mit einer
	    // höheren Wahrscheinlichkeit, als Schneeflocken in der Mitte.
	    if (Math.random() * width / 2 < Math.abs(width / 2 - flakes.get(i).getX())) {

		if (flakes.get(i).melt()) {
		    this.flakes.remove(i);
		    addFlake(false);
		}

	    }

	}

    }

    /**
     * Entfernt eine Schneeflocke und ersetzt sie durch eine neue.
     * 
     * @param snowflake
     *            Schneeflocke, die ersetzt wird
     */
    public void revive(Snowflake snowflake) {
	this.flakes.remove(snowflake);
	this.addFlake(false);
    }

    /**
     * Gibt Schneeflocke an angegebener Position zurück.
     * 
     * @param x
     *            Position in x-Richtung
     * @param y
     *            Position in y-Richtung
     * @return Schneeflocke an gegebener Position oder null.
     */
    public Snowflake snowflakeAt(int x, int y) {

	for (Snowflake s : flakes) {
	    if (s.getX() == x && s.getY() == y) {
		return s;
	    }
	}
	return null;

    }

}
